package com.ekingwin.bas.cloud.exam.web;

import com.ekingwin.bas.cloud.exam.dto.QuerySubjectVO;
import com.ekingwin.bas.cloud.exam.dto.SubjectVerifyVO;
import com.ekingwin.bas.cloud.exam.dto.entity.Subject;
import com.ekingwin.bas.cloud.exam.service.SubjectService;
import com.ekingwin.bas.cloud.infra.utils.common.JsonUtil;
import com.ekingwin.bas.cloud.test.BaseWebTestService;
import com.ekingwin.bas.cloud.test.util.HttpTestUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Mockito;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;

/**
 * @author chenjb
 * @version 1.0.0
 * @ClassName SubjectControllerTest.java
 * @Description TODO
 * @createTime 2021/6/10 15:13
 */
@WebMvcTest(controllers = SubjectController.class)
class SubjectControllerTest extends BaseWebTestService {

    //@ParameterizedTest注解表示参数化测试，可以通过@ValueSource注解自定义测试时需要用到的参数值
    //@ValueSource只支持同时传入一个参数，虽然定义的类型是数组，那也只是根据数组中的元素个数作为入参进行多次测试
    //动态的参数支持java的8中基本类型的数组外，也支持class对象数组（不是对象实例）
    @ParameterizedTest
    @ValueSource(strings = {"s", "e"})
    @DisplayName("添加科目")
    void addSubject(String param) throws Exception {
        //测试web层时，先构建一个http测试工具
        HttpTestUtils http = super.build();

        //构建请求参数（测试的目标函数逻辑中，对与参数的属性并没有要求，所以这里定义一个空的实例就行）
        Subject subject = new Subject();

        //函数逻辑中，调用了底层业务类中的两个方法；这两个方法不需要再这里进行测试（会有单独的单元测试类），因此通过stub进行模拟返回
        SubjectVerifyVO subjectVerify = new SubjectVerifyVO();
        int result = 0;
        int code = 200;
        //根据动态参数，设置不同的模拟返回值
        if ("s".equals(param)) {
            subjectVerify.setMsg("success");
            subjectVerify.setSuccess(true);
            result = 1;
        } else {
            subjectVerify.setMsg("error");
            subjectVerify.setSuccess(false);
            code = 400;
        }
        //开始打桩（stub）
        Mockito.doReturn(subjectVerify).when(subjectService).verify(any(Subject.class));
        Mockito.doReturn(result).when(subjectService).addSubject(any(Subject.class));

        //将请求参数转为string字符串（一般为json字符串）
        String content = JsonUtil.objectToJsonString(subject);
        //除了requestBody的传参方式，也支持requestParam的方式
        MultiValueMap<String, String> params = new LinkedMultiValueMap<>();

        //通过http测试工具发送请求
        //参数也支持通过url传递，如/addSubject/{id}
        ResultActions resultActions = http.post("/addSubject", content, params, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(code));

        //也可以获取最终响应值，在来进行断言
        MvcResult mvcResult = resultActions.andReturn();
        //通过断言工具，对获取到的响应信息进行校验
        //如果不需要对返回结果的业务数据进行校验，也可以省略这一步
//        Assertions.assertEquals(code, mvcResult.getResponse().getStatus());
    }

    //这个测试的目标方法中的逻辑比较简单，其实是可以不用测试的，不过对于controller，目前都需要做定义一个单元测试，主要目的是保证接口路径的准确性；
    //另外，controller中的逻辑基本都十分简单，所以一般不需要对其中的逻辑进行测试
    @Test
    @DisplayName("删除科目")
    void dleSubjectById() throws Exception {
        HttpTestUtils http = super.build();
        //我们只测试目标方法的逻辑，它所引用的底层方法我们这里并不关注，模拟一个假的返回值即可
        //实际上，mock的对象调用的方法默认返回的是null，如果在逻辑中不会造成空指针，同时也不需要使用该返回值进行逻辑判断，这里打打桩操作也可以省略
        Mockito.doReturn(1).when(subjectService).delSubjectById(anyString());

        //定义controller的web参数
        MultiValueMap<String, String> params = new LinkedMultiValueMap<>();
        List<String> idList = new ArrayList<>();
        idList.add("1");
        params.put("ids", idList);

        //发送模拟请求
        ResultActions resultActions = http.get("/delSubjectById", null, params, null);
        //验证是否是想要得到的记过
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));
    }

    @Test
    @DisplayName("查询科目")
    void querySubjectById() throws Exception {
        HttpTestUtils http = super.build();
        QuerySubjectVO query = new QuerySubjectVO();
        String content = JsonUtil.objectToJsonString(query);

        ResultActions resultActions = http.post("/querySubject", content, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(200));

        //也可以获取最终响应值，在来进行断言
        MvcResult mvcResult = resultActions.andReturn();
    }

    @Test
    @DisplayName("修改科目信息")
    void changeSubjectInfo() throws Exception {
        HttpTestUtils http = super.build();
        Subject subject = new Subject();
        String content = JsonUtil.objectToJsonString(subject);

        Mockito.doReturn(new SubjectVerifyVO()).when(subjectService).verify(any(Subject.class));
        Mockito.doReturn(1).when(subjectService).addSubject(any(Subject.class));

        ResultActions resultActions = http.post("/changeSubjectInfo", content, null, null);

        //可以为虚拟结果添加预期值（这里支持jsonpath，可以根据json路径获取值来进行判断）
        resultActions.andExpect(MockMvcResultMatchers.jsonPath("$.code").value(400));

        //也可以获取最终响应值，在来进行断言
        MvcResult mvcResult = resultActions.andReturn();
    }

    @Override
    protected String getBaseUrl() {
        return "/exam/subject";
    }

    @SpringBootApplication
//    @SpringBootApplication(scanBasePackages = {"com.ekingwin.bas.cloud.**.web"})
    static class InnerConfig {
    }

    @MockBean
    private SubjectService subjectService;
}